@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: dispatch.c 1992 2006-08-22 12:48:31Z schoenw $
+ * $Id: dispatch.c 3178 2009-07-28 10:23:24Z schoenw $
  */
 
 #include "gsnmp.h"
@@ -37,8 +37,7 @@
  */
 
 gboolean
-gnet_snmp_dispatcher_send_pdu(GNetSnmpTDomain tDomain,
-			      GInetAddr *tAddress,
+gnet_snmp_dispatcher_send_pdu(GNetSnmpTAddress *tAddress,
 			      GNetSnmpVersion version,
 			      GNetSnmpSecModel sec_model,
 			      GString *sec_name,
@@ -101,7 +100,7 @@ gnet_snmp_dispatcher_send_pdu(GNetSnmpTDomain tDomain,
     if (community) g_free(community);
     gnet_snmp_ber_enc_delete(ber, &start, &len);
     
-    if (!gnet_snmp_transport_send(tDomain, tAddress, start, len, error))
+    if (!gnet_snmp_transport_send(tAddress, start, len, error))
 	return FALSE;
 
     return TRUE;
@@ -112,8 +111,7 @@ gnet_snmp_dispatcher_send_pdu(GNetSnmpTDomain tDomain,
  */
 
 gboolean
-gnet_snmp_dispatcher_recv_msg(GNetSnmpTDomain tDomain,
-			      GInetAddr *tAddress,
+gnet_snmp_dispatcher_recv_msg(GNetSnmpTAddress *tAddress,
 			      guchar *buffer,
 			      gsize buffer_len,
 			      GError **error)
