@@ -19,7 +19,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc.,  59 Temple Place - Suite 330, Cambridge, MA 02139, USA.
  *
- * $Id: utils.c 1992 2006-08-22 12:48:31Z schoenw $
+ * $Id: utils.c 3257 2010-02-17 12:17:35Z schoenw $
  */
 
 #include "gsnmp.h"
@@ -302,9 +302,11 @@ gnet_snmp_attr_assign(GList *vbl,
 	     if (gp) *gp = (gpointer) vb->value.ui8v;
 	     break;
 	 case GNET_SNMP_VARBIND_TYPE_OBJECTID:
+	     if (gp) *gp = (gpointer) vb->value.ui32v;
+	     break;
 	 case GNET_SNMP_VARBIND_TYPE_IPADDRESS:
 	 case GNET_SNMP_VARBIND_TYPE_OPAQUE:
-	     *gp = (gpointer) vb->value.ui32;
+	     if (gp) *gp = (gpointer) vb->value.ui8v;
 	     break;
 	 case GNET_SNMP_VARBIND_TYPE_COUNTER64:
 	     if (gp) *gp = (gpointer) &(vb->value.ui64);
@@ -312,7 +314,8 @@ gnet_snmp_attr_assign(GList *vbl,
 	 default:
 	     break;
 	 }
-	 if (gp && attributes[i].len_offset) {
+	 if (gp && attributes[i].len_offset
+	     && (!(attributes[i].flags & GSNMP_ATTR_FLAG_FIXED_LENGTH))) {
 	     guint16 *lp;
 	     lp = (guint16 *) G_STRUCT_MEMBER_P(p, attributes[i].len_offset);
 	     *lp = vb->value_len;
@@ -383,7 +386,11 @@ gnet_snmp_attr_set(const GNetSnmp *s, GList **vbl,
 	}
 	gp = G_STRUCT_MEMBER_P(p, attributes[i].val_offset);
 	if (attributes[i].len_offset) {
-	    lp = (guint16 *) G_STRUCT_MEMBER_P(p, attributes[i].len_offset);
+            if (attributes[i].flags & GSNMP_ATTR_FLAG_FIXED_LENGTH) {
+                lp = (guint16 *) &attributes[i].len_offset;
+	    } else {
+		lp = (guint16 *) G_STRUCT_MEMBER_P(p, attributes[i].len_offset);
+	    }
 	} else {
 	    lp = 0;
 	}
@@ -419,7 +426,7 @@ gnet_snmp_compare_oids(const guint32 *oid1, const gsize len1,
 
 
 GURI*
-gnet_snmp_parse_uri(const gchar *uri_string)
+gnet_snmp_parse_uri(const gchar *uri_string, GError **error)
 {
     GURI *uri;
     gchar *string;
@@ -435,15 +442,33 @@ gnet_snmp_parse_uri(const gchar *uri_string)
        (which still would have to be written as "[::1]"). */
     
     uri = gnet_uri_new(string);
-    if (uri && !uri->scheme && !uri->hostname) {
-	gnet_uri_delete(uri);
-	uri = NULL;
-    }
-    if (uri && strcmp(uri->scheme, "snmp") != 0) {
+
+    if (uri && uri->scheme
+	&& (strcmp(uri->scheme, "snmp") != 0 && strcmp(uri->scheme, "file") != 0)) {
 	gnet_uri_delete(uri);
 	return NULL;
     }
-    
+
+    if (uri && uri->scheme && strcmp(uri->scheme, "snmp") == 0 && !uri->hostname) {
+	gnet_uri_delete(uri);
+	uri = NULL;
+    }
+
+    if (uri && ! uri->scheme && strchr(string, '/')) {
+	uri = gnet_uri_new_fields_all("file", NULL, NULL, 0,
+				      string, NULL, NULL);
+    }
+
+    /*
+     * Kill the uri parsing state if we still do not have a
+     * scheme so that we get a URI assembled manually below.
+     */
+
+    if (uri && ! uri->scheme) {
+	gnet_uri_delete(uri);
+	uri = NULL;
+    }
+
     if (! uri) {
 	gchar *hostname = NULL, *userinfo = NULL, *port = NULL;
 	
@@ -451,7 +476,7 @@ gnet_snmp_parse_uri(const gchar *uri_string)
 	if (hostname) {
 	    userinfo = string;
 	    *hostname = 0;
-	    hostname++;
+		hostname++;
 	} else {
 	    hostname = string;
 	}
@@ -464,14 +489,14 @@ gnet_snmp_parse_uri(const gchar *uri_string)
 	
 	uri = gnet_uri_new_fields_all("snmp", userinfo, hostname,
 				      port ? atoi(port) : 161,
-				      "", NULL, NULL);
+					  "", NULL, NULL);
     }
 
     if (uri && !uri->userinfo) {
 	gnet_uri_set_userinfo(uri, "public");
     }
 
-    if (uri && uri->port == 0) {
+    if (uri && uri->scheme && strcmp(uri->scheme, "snmp") == 0 && uri->port == 0) {
 	gnet_uri_set_port(uri, 161);
     }
 
@@ -746,7 +771,7 @@ static GOptionEntry gsnmp_args[] = {
     { "snmp-retries", 0, 0, G_OPTION_ARG_INT, &gnet_snmp_retries,
       "Set the default retries to N", "N" },
     { "snmp-version", 0, 0, G_OPTION_ARG_CALLBACK, opt_version_cb,
-      "Set the default SNMP version to V", "V" },
+      "Set the default SNMP version to V (default SNMPv2c)", "V" },
     { "snmp-transport", 0, 0, G_OPTION_ARG_NONE, NULL,
       "Set the default SNMP transport domain to T", "T" },
     { NULL },
